// Protocol Buffers - Google's data interchange format
// Copyright 2026 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2024";

package pb;

// This file is the user facing cpp language proto that bundles all descriptor
// type cpp language options.

import public "google/protobuf/cpp_file_options.proto";
